// $Id$

/**
 * Activate a module and any modules it depends on.
 */
Drupal.contribToggleActivate = function (title, modules) {
  modules[title].self.attr('checked', true);
  jQuery.each(modules[title].dependsOn, function(i, val) {
    Drupal.contribToggleActivate(val, modules);
  });
}

/**
 * Deactivate a module and all the modules that depend on it.
 */
Drupal.contribToggleDeactivate = function (title, modules) {
  modules[title].self.attr('checked', false);
  jQuery.each(modules[title].requiredBy, function(i, val) {
    Drupal.contribToggleDeactivate(val, modules);
  });
}

/**
 * Parse a module's Required by, Depends on, or Missing strings.
 *
 * @param: str - e.g. 'Required by: Event Views (disabled), Views UI (disabled)'
 * @return: array - e.g. ['Event Views', 'Views UI']
 */
Drupal.contribToggleParse = function (str) {
  var result = [];
  if (str) {
    str = str.substr(str.indexOf(':')+1); // Remove 'Required by:'
    str = str.replace( /\([^)]*\)/g, ''); // Remove (stuff).
    result = jQuery.map(str.split(','), function(n,i) {
      return jQuery.trim(n);
    });
  }
  return result;
}

/**
 * Determine if releated modules are active.
 *
 * @param: title - e.g. 'Contrib Toggle'
 * @param: relationship - requiredBy or dependsOn
 * @return:
 */
Drupal.contribToggleDependsOnInactive = function (title, modules) {
  var modulesToToggle = [];
  jQuery.each(modules[title].dependsOn, function(i, val) {
    if (!modules[val].self.attr('checked')) {
        modulesToToggle[modulesToToggle.length] = val;
    }
  });
  return modulesToToggle;
}

Drupal.contribToggleRequiredByActive = function (title, modules) {
  var modulesToToggle = [];
  jQuery.each(modules[title].requiredBy, function(i, val) {
    if (modules[val].self.attr('checked')) {
        modulesToToggle[modulesToToggle.length] = val;
    }
  });
  return modulesToToggle;
}

/**
 * Disable all non-core modules.
 */
Drupal.contribToggleSwitch = function () {
  // Read dependencies for all modules.
  var modules = [];
  $('#system-modules input[@type=checkbox][@name^="status"]').each( function() {
    var $row = $(this).parents('tr:first');
	var $title = $row.find('label:last').text();
	var $dependsOn = Drupal.contribToggleParse($row.find('.admin-dependencies').text());
	var $requiredBy = Drupal.contribToggleParse($row.find('.admin-required').text());
	modules[$title] = { self: $(this), dependsOn : $dependsOn, requiredBy : $requiredBy };
  });
  // Disable each non-core module, and any modules that depend on it.
  $('#system-modules input[@type=checkbox][@name^="status"]').each( function() {
    // Identify package that module belongs to.
    var fieldsetLegend = $(this).parents('fieldset').find('legend').text();
    // Mark to be disabled all modules not in packages listed as cases.
    switch (fieldsetLegend) {
      case "Core - optional":
        break;
        
      case "Core - required":
        break;

      default:
        var title = $(this).parents('tr:first').find('label:last').text();
        Drupal.contribToggleDeactivate(title, modules);
    }
  });
}

if( Drupal.jsEnabled ) {
  $(function() { 
	var modules = [];
    $('#system-modules input[@type=checkbox][@name^="status"]').each( function() {
      // Read dependencies for all modules.
	  var $row = $(this).parents('tr:first');
	  var $title = $row.find('label:last').text();
	  var $dependsOn = Drupal.contribToggleParse($row.find('.admin-dependencies').text());
	  var $requiredBy = Drupal.contribToggleParse($row.find('.admin-required').text());
      var $missing = Drupal.contribToggleParse($row.find('.admin-missing').text());
	  modules[$title] = { self: $(this), dependsOn : $dependsOn, requiredBy : $requiredBy, missing : $missing };

      // If module has any missing requirements, set checkbox to not checked and disabled
      // and move to next checkbox
      if (modules[$title].missing.length > 0) {
          $(this).attr('checked', false);
          $(this).attr('disabled', true);
          // Returning true moves to next checkbox, bypassing the code below that would otherwise
          // enable the checkbox, but continues with overall loop.
          return true;
      }

      // Identify package that module belongs to.
      var fieldsetLegend = $(this).parents('fieldset').find('legend').text();
      
      // Enable checkboxes for all modules, even if they have dependencies.
      switch (fieldsetLegend) {
        // Keep checkboxes disabled for Core - required modules.
        case "Core - required":
          break;
        default:
          $(this).attr('disabled', false);
        };
      });
	
	$('#system-modules input[@type=checkbox][@name^="status"]').click( function() {
	  var $title = $(this).parents('tr:first').find('label:last').text();
	  var checked = $(this).attr('checked');
	  if (checked) {
        $(this).attr('checked', false);
        if (modules[$title].dependsOn.length > 0) {
          var modulesToActivate = [];
          modulesToActivate = Drupal.contribToggleDependsOnInactive($title, modules);
          if (modulesToActivate.length > 0) {
            var $modulePlural = "module";
            if (modulesToActivate.length > 1) {
              $modulePlural = $modulePlural + "s";
            }
            $confirmMessage = $title + ' requires the following ' + $modulePlural + ' to be activated as well: <br/>&nbsp;&nbsp;&nbsp;<strong>'
              + modulesToActivate.join('<br />&nbsp;&nbsp;&nbsp;') 
              + '</strong><br />&nbsp;<br />Do you wish to proceed?';
            jConfirm($confirmMessage, 'Required ' + $modulePlural, function(proceed) {
              if (proceed) {
                Drupal.contribToggleActivate($title, modules);
                }
            });
          }
          else {
            Drupal.contribToggleActivate($title, modules);
          }
        }
        else {
          Drupal.contribToggleActivate($title, modules);
        }
	  }
	  else {
        $(this).attr('checked', true);
        if (modules[$title].requiredBy.length > 0) {
          var modulesToDeactivate = [];
          modulesToDeactivate = Drupal.contribToggleRequiredByActive($title, modules);
          if (modulesToDeactivate.length > 0) {
            var $modulePlural = "module";
            if (modulesToDeactivate.length > 1) {
              $modulePlural = $modulePlural + "s";
            }
            $confirmMessage = $title + ' is required by the following ' + $modulePlural + ', which would also need to be deactivated: <br/>&nbsp;&nbsp;&nbsp;<strong>'
              + modulesToDeactivate.join('<br />&nbsp;&nbsp;&nbsp;') 
              + '</strong><br />&nbsp;<br />Do you wish to proceed?';
            jConfirm($confirmMessage, 'Dependent ' + $modulePlural, function(proceed) {
              if (proceed) {
                Drupal.contribToggleDeactivate($title, modules);
                }
            });
          }
          else {
            Drupal.contribToggleDeactivate($title, modules);
          }
        }
        else {
          Drupal.contribToggleDeactivate($title, modules);
        }
	  }
    });
  });
}